/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifndef _Omni
#define _Omni

#include "Device.hpp"
#include "DeviceInfo.hpp"
#include "OmniPDCProxy.hpp"
#include "OmniDevice.hpp"

#include <glib.h>
#include <gmodule.h>

extern PSZCRO            vapszLibraryPaths[];
extern PSZCRO            vapszDataPaths[];

class Omni
{
public:
   static void              initialize                  ();
   static void              terminate                   ();
   static int               my_system                   (PSZCRO             pszCommand);

   static void              setOutputStream             (Device            *pDevice,
                                                         FILE              *pfpNew);
   static void              setErrorStream              (Device            *pDevice,
                                                         FILE              *pfpNew);

   static bool              libraryValid                (GModule           *hDevice,
                                                         PSZCRO             pszLibraryName,
                                                         PSZCRO             pszVersion,
                                                         bool               fVerbose = false);

   static bool              openLibrary                 (PSZCRO             pszLibName,
                                                         GModule          **phLibrary);

   static bool              openDeviceLibrary           (PSZCRO             pszLibName,
                                                         GModule          **phDevice);

   static Device           *createDevice                (PSZCRO             pszDriver,
                                                         PSZCRO             pszDevice,
                                                         PSZCRO             pszJobProperties,
                                                         bool               fAdvanced,
                                                         GModule          **phDevice);

   static Device           *createDevice                (PDL               *pPDL,
                                                         PSZCRO             pszJobProperties,
                                                         bool               fAdvanced,
                                                         GModule          **phDevice);

   static DeviceInfo       *findDeviceInfoFromShortName (PSZCRO             pszShortName);

   static PSZCRO            openXMLFile                 (PSZCRO             pszXMLFile);

   static Enumeration      *listDevices                 (bool               fBuildOnly = false);
   static Enumeration      *listXMLDevices              (bool               fBuildOnly = false);
};

#endif
