/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include "Star_ESC_Instance.hpp"
#include "defines.hpp"

#include <iostream>
#include <sstream>

const static bool fTestNoCompression = true;

DeviceInstance *
createInstance (PrintDevice *pDevice)
{
   return new Star_ESC_Instance (pDevice);
}

void
deleteInstance (DeviceInstance *pInstance)
{
   delete pInstance;
}

Star_ESC_Instance::
Star_ESC_Instance (PrintDevice *pDevice)
   : DeviceInstance (pDevice)
{
#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "Star_ESC_Instance::Star_ESC_Instance ()" << std::endl;
#endif

   fHaveInitialized_d  = false;
   fHaveSetupPrinter_d = false;
   fUseBidirectional_d = true;
   ptlPrintHead_d.x    = 0;
   ptlPrintHead_d.y    = 0;
}

Star_ESC_Instance::
~Star_ESC_Instance ()
{
#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "Star_ESC_Instance::~Star_ESC_Instance ()" << std::endl;
#endif
}

void Star_ESC_Instance::
initializeInstance ()
{
   if (fHaveInitialized_d)
      return;

   fHaveInitialized_d = true;

#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "Star_ESC_Instance::initializeInstance ()" << std::endl;
#endif
}

static PSZRO apszDeviceJobPropertyKeys[] = {
   "bidirectional"
};
#define JOBPROP_BIDIRECTIONAL apszDeviceJobPropertyKeys[0]

static PSZRO apszDeviceJobPropertyValues[] = {
   "true",
   "false"
};
#define JOBPROP_TRUE    apszDeviceJobPropertyValues[0]
#define JOBPROP_FALSE   apszDeviceJobPropertyValues[1]

class DeviceJobPropertyKeyEnumerator : public Enumeration
{
public:
   DeviceJobPropertyKeyEnumerator (PSZRO *apszKeys, int iDimOfKeys)
   {
      iWhichKey_d = iDimOfKeys;
      apszKeys_d  = apszKeys;
   }

   virtual bool hasMoreElements ()
   {
      return iWhichKey_d > 0;
   }

   virtual void * nextElement ()
   {
      if (0 < iWhichKey_d)
      {
         iWhichKey_d--;

         return (void *)apszKeys_d[iWhichKey_d];
      }

      return 0;
   }

private:
   int    iWhichKey_d;
   PSZRO *apszKeys_d;
};

Enumeration * Star_ESC_Instance::
listDeviceJobPropertyKeys ()
{
   return new DeviceJobPropertyKeyEnumerator (apszDeviceJobPropertyKeys, dimof (apszDeviceJobPropertyKeys));
}

class KeyEnumerator : public Enumeration
{
public:
   KeyEnumerator (PSZRO pszKeys)
   {
      pszKeys_d    = 0;
      pszCurrent_d = 0;

      if (  pszKeys
         && *pszKeys
         )
      {
         pszKeys_d = (char *)malloc (strlen (pszKeys) + 1);
         if (pszKeys_d)
         {
            strcpy (pszKeys_d, pszKeys);
            pszCurrent_d = pszKeys_d;
         }
      }
   }

   virtual ~KeyEnumerator ()
   {
      if (pszKeys_d)
      {
         free (pszKeys_d);
         pszKeys_d = 0;
      }
   }

   virtual bool hasMoreElements ()
   {
      return pszCurrent_d ? true : false;
   }

   virtual void *nextElement ()
   {
      void *pvRet = pszCurrent_d;

      if (  pszCurrent_d
         && *pszCurrent_d
         )
      {
         char *pszSpace = strchr (pszCurrent_d, ' ');

         if (pszSpace)
         {
            *pszSpace = '\0';
            pszCurrent_d = pszSpace + 1;
            while (' ' == *pszCurrent_d)
            {
               pszCurrent_d++;
            }
            if (!*pszCurrent_d)
            {
               pszCurrent_d = 0;
            }
         }
         else
         {
            pszCurrent_d = 0;
         }
      }
      else
      {
         pszCurrent_d = 0;
      }

      return pvRet;
   }

private:
   char *pszKeys_d;
   char *pszCurrent_d;
};

Enumeration * Star_ESC_Instance::
listKeyValues (PSZRO pszKey)
{
   std::ostringstream oss;

   if (0 == strcasecmp (pszKey, JOBPROP_BIDIRECTIONAL))
   {
      oss << JOBPROP_TRUE
          << " "
          << JOBPROP_FALSE;
   }

   return new KeyEnumerator (oss.str ().c_str ());
}

bool Star_ESC_Instance::
handleDeviceJobProperty (PSZRO pszKey,
                         PSZRO pszValue)
{
#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "Star_ESC_Instance::handleDeviceJobProperty ("
                          << pszKey << ", "
                          << pszValue << ")"
                          << std::endl;
#endif

   if (0 == strcasecmp (pszKey, JOBPROP_BIDIRECTIONAL))
   {
      if (0 == strcmp (pszValue, JOBPROP_TRUE))
      {
         fUseBidirectional_d = true;
         return true;
      }
      else if (0 == strcmp (pszValue, JOBPROP_FALSE))
      {
         fUseBidirectional_d = false;
         return true;
      }
   }

   return false;
}

std::string * Star_ESC_Instance::
getJobPropertyType (PSZRO pszKey)
{
   std::string *pRet = 0;

   if (0 == strcasecmp (pszKey, JOBPROP_BIDIRECTIONAL))
   {
      pRet = new std::string ("boolean ");
      *pRet += JOBPROP_TRUE;
   }

   return pRet;
}

std::string * Star_ESC_Instance::
getJobProperty (PSZRO pszKey)
{
   if (0 == strcasecmp (pszKey, JOBPROP_BIDIRECTIONAL))
   {
      if (fUseBidirectional_d)
      {
         return new std::string (JOBPROP_TRUE);
      }
      else
      {
         return new std::string (JOBPROP_FALSE);
      }
   }

   return 0;
}

std::string * Star_ESC_Instance::
translateKeyValue (PSZRO pszKey,
                   PSZRO pszValue)
{
   PSZRO        pszRetKey   = 0;
   PSZRO        pszRetValue = 0;
   std::string *pRet        = 0;
   int          iValueEnum  = StringResource::DEVICE_COMMON_UNKNOWN;

   if (0 == strcasecmp (pszKey, JOBPROP_BIDIRECTIONAL))
   {
      pszRetKey = StringResource::getString (pDevice_d->getLanguageResource (),
                                             StringResource::STRINGGROUP_DEVICE_COMMON,
                                             StringResource::DEVICE_COMMON_BIDIRECTIONAL);
   }

   if (pszValue)
   {
      if (0 == strcasecmp (pszValue, JOBPROP_TRUE))
         iValueEnum = StringResource::DEVICE_COMMON_TRUE;
      else if (0 == strcasecmp (pszValue, JOBPROP_FALSE))
         iValueEnum = StringResource::DEVICE_COMMON_FALSE;
   }

   if (StringResource::DEVICE_COMMON_UNKNOWN != iValueEnum)
   {
      pszRetValue = StringResource::getString (pDevice_d->getLanguageResource (),
                                               StringResource::STRINGGROUP_DEVICE_COMMON,
                                               iValueEnum);
   }

   if (pszRetKey)
   {
      pRet = new std::string (pszRetKey);
   }

   if (  pszRetValue
      && pRet
      )
   {
      *pRet += "=";
      *pRet += pszRetValue;
   }

   return pRet;
}

void Star_ESC_Instance::
setupPrinter ()
{
   if (fHaveSetupPrinter_d)
      return;

#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "Star_ESC_Instance::setupPrinter ()" << std::endl;
#endif

   fHaveSetupPrinter_d = true;

   DeviceResolution *pRes      = getCurrentResolution ();
   DeviceForm       *pDF       = getCurrentForm ();
   HardCopyCap      *pHCC      = pDF->getHardCopyCap ();
   DeviceCommand    *pCommands = getCommands ();
   BinaryData       *pCmd      = 0;

   pCmd = pCommands->getCommandData ("cmdBidi");
   if (pCmd)
   {
      char chState = '0';

      if (fUseBidirectional_d)
      {
         chState = '1';
      }

      pDevice_d->sendPrintfToDevice (pCmd, chState);
   }
   else
   {
#ifndef RETAIL
      if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "Error: There is no cmdBidi defined for this device!" << std::endl;
#endif
   }

   // @TBD
///pDevice_d->sendBinaryDataToDevice (getCurrentTray ());

   pCmd = pCommands->getCommandData ("cmdSetColor");
   if (pCmd)
   {
      // Start with black ink (default)
      pDevice_d->sendPrintfToDevice (pCmd, '0');
   }

   int iLines = (int)(6.0 * (float)pHCC->getCy () / 25400.0);

#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "iLines = " << iLines << std::endl;
#endif

   pCmd = pCommands->getCommandData ("cmdSetSixthLineSpacing");
   if (pCmd)
   {
      pDevice_d->sendBinaryDataToDevice (pCmd);
   }

   pCmd = pCommands->getCommandData ("cmdSetPageLengthInLines");
   if (pCmd)
   {
      pDevice_d->sendPrintfToDevice (pCmd, iLines);
   }

   int iColumnSize = pRes->getScanlineMultiple ();
   int iSpacing    = 0;

   pCmd = pCommands->getCommandData ("cmdSetLineSpacing216inch");
   if (pCmd)
   {
      iSpacing = 216;
   }
   else
   {
      pCmd = pCommands->getCommandData ("cmdSetLineSpacing180inch");
      if (pCmd)
      {
         iSpacing = 180;
      }
   }

#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << *pCmd << " iSpacing = " << iSpacing << ", iColumnSize = " << iColumnSize << ", pRes->getYRes () = " << pRes->getYRes () << std::endl;
#endif

   if (0 < iSpacing)
   {
      pDevice_d->sendPrintfToDevice (pCmd,
                                     iSpacing * iColumnSize / pRes->getYRes ());
   }
}

bool Star_ESC_Instance::
beginJob ()
{
   DeviceCommand    *pCommands = getCommands ();
   BinaryData       *pCmd      = 0;

#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "Star_ESC_Instance::beginJob ()" << std::endl;
#endif

   pCmd = pCommands->getCommandData ("cmdInit");
   if (pCmd)
   {
#ifndef RETAIL
      if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "cmdInit = " << *pCmd << std::endl;
#endif

      pDevice_d->sendBinaryDataToDevice (pCmd);
   }

   return true;
}

bool Star_ESC_Instance::
beginJob (bool fJobPropertiesChanged)
{
#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "Star_ESC_Instance::beginJob (with props)" << std::endl;
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "The job properties are as follows:" << std::endl;
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "The orientation is = " << getCurrentOrientation() << std::endl;
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "iDitherID = " << getCurrentDitherID () << std::endl;
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "The form is = " << getCurrentForm () << std::endl;
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "The tray is = " << getCurrentTray () << std::endl;
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "The media is = " << getCurrentMedia () << std::endl;
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "The resolution is = " << getCurrentResolution () << std::endl;
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "The print mode is = " << getCurrentPrintMode () << std::endl;
#endif

   // @TBD - reinitialize with new job properties

   // Call common code
   return beginJob ();
}

bool Star_ESC_Instance::
newFrame ()
{
   DeviceCommand    *pCommands = getCommands ();
   BinaryData       *pCmd      = 0;

#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "Star_ESC_Instance::newFrame ()" << std::endl;
#endif

   pCmd = pCommands->getCommandData ("cmdPageEject");
   if (pCmd)
   {
#ifndef RETAIL
      if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "cmdPageEject = " << *pCmd << std::endl;
#endif

      pDevice_d->sendBinaryDataToDevice (pCmd);
   }

   ditherNewFrame ();

   ptlPrintHead_d.x = 0;
   ptlPrintHead_d.y = 0;

   return true;
}

bool Star_ESC_Instance::
newFrame (bool fJobPropertiesChanged)
{
#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "Star_ESC_Instance::newFrame (with props)" << std::endl;
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "The job properties are as follows:" << std::endl;
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "The orientation is = " << getCurrentOrientation() << std::endl;
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "iDitherID = " << getCurrentDitherID () << std::endl;
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "The form is = " << getCurrentForm () << std::endl;
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "The tray is = " << getCurrentTray () << std::endl;
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "The media is = " << getCurrentMedia () << std::endl;
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "The resolution is = " << getCurrentResolution () << std::endl;
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "The print mode is = " << getCurrentPrintMode () << std::endl;
#endif

   // @TBD - reinitialize with new job properties

   // Call common code
   return newFrame ();
}

bool Star_ESC_Instance::
endJob ()
{
   DeviceCommand    *pCommands = getCommands ();
   BinaryData       *pCmd      = 0;

#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "Star_ESC_Instance::endJob ()" << std::endl;
#endif

   pCmd = pCommands->getCommandData ("cmdPageEject");
   if (pCmd)
   {
#ifndef RETAIL
      if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "cmdPageEject = " << *pCmd << std::endl;
#endif

      pDevice_d->sendBinaryDataToDevice (pCmd);
   }

   pCmd = pCommands->getCommandData ("cmdTerm");
   if (pCmd)
   {
#ifndef RETAIL
      if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "cmdTerm = " << *pCmd << std::endl;
#endif

      pDevice_d->sendBinaryDataToDevice (pCmd);
   }

   return true;
}

bool Star_ESC_Instance::
abortJob ()
{
#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) DebugOutput::getErrorStream () << "Star_ESC_Instance::endJob ()" << std::endl;
#endif

   int cx = getCurrentForm ()->getHardCopyCap ()->getXPels ();

   // The largest size is 24 lines
   int iSize = 24 * (cx + 7) / 8;

   PBYTE abData = new BYTE [iSize];

   memset (abData, 0, sizeof (abData));

   BinaryData data (abData, sizeof (abData));

   pDevice_d->sendBinaryDataToDevice (&data);

   delete []abData;

   return true;
}

std::string Star_ESC_Instance::
toString (std::ostringstream& oss)
{
   oss << "{ "
       << DeviceInstance::toString (oss) // Add parent's output as well
       << " }";

   return oss.str ();
}

/* Provide a way to print out class data
*/
std::ostream&
operator<< (std::ostream& os, const Star_ESC_Instance& const_self)
{
   Star_ESC_Instance& self = const_cast<Star_ESC_Instance&>(const_self);
   std::ostringstream  oss;

   os << self.toString (oss);

   return os;
}
