/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifndef _XMLDevice
#define _XMLDevice

#include <Device.hpp>
#include <OmniProxy.hpp>

#include <map>

#include <libxml/parser.h>
#include <libxml/tree.h>

extern "C" {
   PSZCRO       getVersion                  ();
   Enumeration *getDeviceEnumeration        (PSZCRO  pszLibraryName,
                                             bool    fBuildOnly);
   Device      *newDeviceW_Advanced         (bool    fAdvanced);
   Device      *newDeviceW_JopProp_Advanced (PSZCRO  pszJobProperties,
                                             bool    fAdvanced);
   void         deleteDevice                (Device *pDevice);
};

struct XMLDeviceInstance;
struct XMLDeviceBlitter;
struct XMLDevice;

class XMLDevice : public PrintDevice
{
public:
                              XMLDevice                (PSZCRO               pszMasterXML,
                                                        xmlDocPtr            docDevice,
                                                        PSZCRO               pszDriverName,
                                                        PSZCRO               pszDeviceName,
                                                        PSZCRO               pszShortName,
                                                        PSZCRO               pszJobProperties,
                                                        PSZCRO               pszXMLDeviceName);
                             ~XMLDevice                ();

   virtual DeviceBooklet     *getDefaultBooklet        ();
   virtual DeviceCollation   *getDefaultCollation      ();
   virtual DeviceCopies      *getDefaultCopies         ();
   virtual DeviceDestination *getDefaultDestination    ();
   virtual PSZCRO             getDefaultDitherID       ();
   virtual DeviceDuplex      *getDefaultDuplex         ();
   virtual DeviceForm        *getDefaultForm           ();
   virtual DeviceJogging     *getDefaultJogging        ();
   virtual DeviceMedia       *getDefaultMedia          ();
   virtual DeviceNUp         *getDefaultNUp            ();
   virtual DeviceOrientation *getDefaultOrientation    ();
   virtual DevicePrintMode   *getDefaultPrintMode      ();
   virtual DeviceResolution  *getDefaultResolution     ();
   virtual DeviceScaling     *getDefaultScaling        ();
   virtual DeviceStapling    *getDefaultStapling       ();
   virtual DeviceTray        *getDefaultTray           ();

   virtual DeviceCommand     *getDefaultCommands       ();
   virtual DeviceData        *getDefaultData           ();
   virtual DeviceString      *getDefaultString         ();

   virtual DeviceGamma       *getCurrentGamma          ();

   bool                       hasDeviceOption          (PSZCRO               pszDeviceOption);

   static XMLDevice          *isAXMLDevice             (Device              *pDevice);

   xmlNodePtr                 findEntryKeyValue        (xmlNodePtr           root,
                                                        PSZCRO               pszKey,
                                                        PSZCRO               pszValue,
                                                        bool                 fDebugOutput);

   xmlDocPtr                  getDeviceXML             (PSZCRO               pszDeviceType);

   xmlDocPtr                  getDocBooklets           ();
   xmlDocPtr                  getDocForms              ();
   xmlDocPtr                  getDocMedias             ();
   xmlDocPtr                  getDocOrientations       ();
   xmlDocPtr                  getDocPrintModes         ();
   xmlDocPtr                  getDocResolutions        ();
   xmlDocPtr                  getDocTrays              ();
   xmlDocPtr                  getDocGammas             ();

   PSZCRO                     getXMLDeviceName         ();

   DeviceInstance            *getDeviceInstance        ();
   DeviceBlitter             *getDeviceBlitter         ();

   static bool                parseBinaryData          (PSZRO                pszData,
                                                        byte               **ppbData,
                                                        int                 *pcbData) throw (std::string *);

   virtual std::string        toString                 (std::ostringstream&  oss);
   friend std::ostream&       operator<<               (std::ostream&        os,
                                                        const XMLDevice&     self);

private:
   void                       initializeDevice         ();

   std::string            *pstringMasterXMLPath_d;
   char              *pszXMLDeviceName_d;

   xmlDocPtr          docDevice_d;
   xmlNodePtr         rootDevice_d;

   xmlDocPtr          docDeviceBooklets_d;
   xmlDocPtr          docDeviceForms_d;
   xmlDocPtr          docDeviceMedias_d;
   xmlDocPtr          docDeviceOrientations_d;
   xmlDocPtr          docDevicePrintModes_d;
   xmlDocPtr          docDeviceResolutions_d;
   xmlDocPtr          docDeviceTrays_d;
   xmlDocPtr          docDeviceGammas_d;
   xmlDocPtr          docDeviceCommands_d;
   xmlDocPtr          docDeviceDatas_d;
   xmlDocPtr          docDeviceStrings_d;

   PSZRO              pszDefaultBooklet_d;
   PSZRO              pszDefaultForm_d;
   PSZRO              pszDefaultMedia_d;
   PSZRO              pszDefaultOrientation_d;
   PSZRO              pszDefaultPrintMode_d;
   PSZRO              pszDefaultResolution_d;
   PSZRO              pszDefaultTray_d;
   PSZRO              pszDefaultDitherID_d;

   PSZRO              pszInstance_d;
   PSZRO              pszBlitter_d;

   DeviceCommand     *pDefaultCommands_d;
   DeviceData        *pDefaultDatas_d;
   DeviceString      *pDefaultStrings_d;

   DeviceInstance    *pInstance_d;
   DeviceBlitter     *pBlitter_d;

   XMLDeviceInstance *pXMLInstance_d;
   XMLDeviceBlitter  *pXMLBlitter_d;

   typedef std::map <std::string, xmlDocPtr> HasMapping;

   HasMapping         hasMap_d;
};

static inline xmlNodePtr
firstNode (xmlNodePtr elm)
{
   if (!elm)
      return 0;

   while (  0 != elm
         && (  XML_COMMENT_NODE == elm->type
            || XML_TEXT_NODE    == elm->type
            )
         )
   {
      // Move to the next node
      elm = elm->next;
   }

   return elm;
}

static inline xmlNodePtr
nextNode (xmlNodePtr elm)
{
   if (!elm)
      return 0;

   do
   {
      // Move to the next node
      elm = elm->next;

   } while (  0 != elm
           && (  XML_COMMENT_NODE == elm->type
              || XML_TEXT_NODE    == elm->type
              )
           );

   return elm;
}

static inline void
printNode (xmlNodePtr elm,
           bool       fDebugOutput)
{
   if (fDebugOutput)
   {
      xmlElemDump (stderr, elm->doc, elm);
   }
}

static inline xmlNodePtr
findEntry (xmlNodePtr root,
           PSZCRO     pszName,
           bool       fShouldDebugOutput)
{
   if (root == 0)
      return 0;

   xmlNodePtr elm = firstNode (root->xmlChildrenNode);

   while (elm != 0)
   {
      if (0 == strcmp ((char *)elm->name, pszName))
      {
         break;
      }

      elm = nextNode (elm);
   }

#ifndef RETAIL
   if (fShouldDebugOutput)
   {
      if (elm)
      {
         DebugOutput::getErrorStream () << __FUNCTION__ << ": Found ";
      }
      else
      {
         DebugOutput::getErrorStream () << __FUNCTION__ << ": Did not find ";
      }
      DebugOutput::getErrorStream () << pszName << std::endl;
   }
#endif

   return elm;
}

static inline PSZCRO
getXMLContentString (xmlNodePtr  root,
                     xmlDocPtr   doc,
                     char       *pszXMLNodeName)
{
   if (pszXMLNodeName)
   {
      root = findEntry (root, pszXMLNodeName, false);
   }

   if (root)
   {
      return (PSZCRO)xmlNodeListGetString (doc, root->xmlChildrenNode, 1);
   }

   return 0;
}

static inline int
getXMLContentInt (xmlNodePtr root,
                  xmlDocPtr  doc,
                  PSZCRO     pszXMLNodeName,
                  bool       fRequired = true,
                  int        iDefault  = 0)
   throw (std::string *)
{
   xmlNodePtr elm = findEntry (root, pszXMLNodeName, false);

   if (elm)
   {
      char *pszString = (char *)xmlNodeListGetString (doc, elm->xmlChildrenNode, 1);

      if (pszString)
      {
         int iReturn  = 0;
         int iSuccess = false;

         if (1 == sscanf (pszString, "%d", &iReturn))
         {
            iSuccess = true;
         }

         free (pszString);

         if (iSuccess)
         {
            return iReturn;
         }

         std::string stringError = "Could not parse \"";

         stringError += pszString;
         stringError += "\"";

         throw new std::string (stringError);
      }
   }

   if (fRequired)
   {
      std::ostringstream oss;

      oss << "Could not find entry \""
          << pszXMLNodeName
          << "\" for root = 0x"
          << std::hex << (int)root << std::dec;

      throw new std::string (oss.str ());
   }
   else
   {
      return iDefault;
   }
}

static inline PSZCRO
getXMLAttribute (xmlNodePtr root,
                 PSZCRO     pszXMLNodeName,
                 PSZCRO     pszXMLAttributeName)
{
   if (pszXMLNodeName)
   {
      root = findEntry (root, pszXMLNodeName, false);
   }

   if (root)
   {
      return (PSZCRO)xmlGetProp (root, (const xmlChar *)pszXMLAttributeName);
   }

   return 0;
}

#endif
