/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include <XMLDeviceForm.hpp>

XMLDeviceForm::
XMLDeviceForm (Device      *pDevice,
               int          id,
               int          iCapabilities,
               BinaryData  *pbdData,
               HardCopyCap *hcInfo,
               xmlNodePtr   node)
   : DeviceForm (pDevice,
                 id,
                 iCapabilities,
                 pbdData,
                 hcInfo)
{
   node_d        = node;
   pszDeviceID_d = 0;
}

XMLDeviceForm::
~XMLDeviceForm ()
{
   if (pszDeviceID_d)
   {
      free ((void *)pszDeviceID_d);
      pszDeviceID_d = 0;
   }
}

DeviceForm * XMLDeviceForm::
createV (int id)
{
   return XMLDeviceForm::create (pDevice_d, id);
}

DeviceForm * XMLDeviceForm::
create (Device *pDevice,
        PSZCRO  pszId)
{
   int         id       = DeviceForm::FORM_UNLISTED;
   std::string stringId = pszId;
   DeviceForm *pDFRet   = 0;
   int         iXLength = 0;
   int         iYLength = 0;

#ifndef RETAIL
   if (DebugOutput::shouldOutputXMLDeviceForm ()) DebugOutput::getErrorStream () << "XMLDeviceForm::" << __FUNCTION__ << ": id = " << id << std::endl;
#endif

   if (std::string::npos != stringId.find (","))
   {
      std::string stringFormId   = stringId.substr (0, stringId.find (","));
      std::string stringFormSize = stringId.substr (stringId.find (",") + 1);

      id = DeviceForm::nameToID (stringFormId.c_str ());

      if (std::string::npos != stringFormId.find (","))
      {
         std::string stringX = stringFormId.substr (stringFormId.find (",") + 1);
         std::string stringY = stringFormId.substr (0, stringFormId.find (","));

         // @TBD
         sscanf (stringX.c_str (), "%d", &iXLength);
         sscanf (stringY.c_str (), "%d", &iYLength);
      }
      else
      {
         sscanf (stringFormSize.c_str (), "%d", &iYLength);
      }
   }
   else
   {
      id = DeviceForm::nameToID (pszId);
   }

   if (DeviceForm::FORM_UNLISTED != id)
   {
      pDFRet = create (pDevice, id);

      if (pDFRet)
      {
         if (iXLength)
         {
            pDFRet->setCx (iXLength);
         }
         if (iYLength)
         {
            pDFRet->setCy (iYLength);
         }
      }
   }

   return pDFRet;
}

DeviceForm * XMLDeviceForm::
create (Device *pDevice, int id)
{
   XMLDevice *pXMLDevice = XMLDevice::isAXMLDevice (pDevice);

   if (!pXMLDevice)
   {
#ifndef RETAIL
      if (DebugOutput::shouldOutputXMLDeviceForm ()) DebugOutput::getErrorStream () << "XMLDeviceForm::" << __FUNCTION__ << ": !pXMLDevice " << std::endl;
#endif

      return 0;
   }

   xmlDocPtr   docDeviceForms  = pXMLDevice->getDocForms ();
   xmlNodePtr  rootDeviceForms = xmlDocGetRootElement (docDeviceForms);
   xmlNodePtr  elmDeviceForm   = 0;
   DeviceForm *pDFRet          = 0;

   if (!rootDeviceForms)
   {
#ifndef RETAIL
      if (DebugOutput::shouldOutputXMLDeviceForm ()) DebugOutput::getErrorStream () << "XMLDeviceForm::" << __FUNCTION__ << ": !rootDeviceForms " << std::endl;
#endif

      return 0;
   }

   elmDeviceForm = firstNode (rootDeviceForms);
   if (!elmDeviceForm)
   {
#ifndef RETAIL
      if (DebugOutput::shouldOutputXMLDeviceForm ()) DebugOutput::getErrorStream () << "XMLDeviceForm::" << __FUNCTION__ << ": !elmDeviceForm " << std::endl;
#endif

      return 0;
   }

   elmDeviceForm = firstNode (elmDeviceForm->children);

   while (  elmDeviceForm
         && !pDFRet
         )
   {
      PSZCRO pszForm = getXMLContentString (elmDeviceForm,
                                            docDeviceForms,
                                            "name");

      if (pszForm)
      {
         int idForm = DeviceForm::nameToID (pszForm);

         if (idForm == id)
         {
            char        *pszComma        = 0;
            PSZRO        pszCapabilities = 0;
            int          iCapabilities   = 0;
            PSZRO        pszCommand      = 0;
            BinaryData  *pbdData         = 0;
            int          iLeft           = 0;
            int          iTop            = 0;
            int          iRight          = 0;
            int          iBottom         = 0;
            HardCopyCap *hcInfo          = 0;
            xmlNodePtr   elmHCC          = 0;

            pszComma = strchr (pszForm, ',');

            try
            {
#ifndef RETAIL
               if (DebugOutput::shouldOutputXMLDeviceForm ()) DebugOutput::getErrorStream () << "XMLDeviceForm::" << __FUNCTION__ << ": Creating \"" << pszForm << "\"" << std::endl;
#endif

               // Read in the capabilities
               pszCapabilities = getXMLContentString (elmDeviceForm, docDeviceForms, "formCapabilities");

               if (pszCapabilities)
               {
                  iCapabilities = DeviceForm::getReservedValue (pszCapabilities);

                  free ((void *)pszCapabilities);
               }

               // Read in the command
               pszCommand = getXMLContentString (elmDeviceForm, docDeviceForms, "command");

               if (pszCommand)
               {
                  byte       *pbData          = 0;
                  int         cbData          = 0;

                  if (XMLDevice::parseBinaryData (pszCommand,
                                                  &pbData,
                                                  &cbData))
                  {
                     pbdData = new BinaryData (pbData, cbData);
                  }

                  free ((void *)pszCommand);
               }

               // Read in the HardCopyCap
               elmHCC  = findEntry (elmDeviceForm, "hardCopyCap", false);
               iLeft   = getXMLContentInt (elmHCC, docDeviceForms, "hardCopyCapLeft");
               iTop    = getXMLContentInt (elmHCC, docDeviceForms, "hardCopyCapTop");
               iRight  = getXMLContentInt (elmHCC, docDeviceForms, "hardCopyCapRight");
               iBottom = getXMLContentInt (elmHCC, docDeviceForms, "hardCopyCapBottom");
               hcInfo  = new HardCopyCap (iLeft, iTop, iRight, iBottom);

#ifndef RETAIL
               if (DebugOutput::shouldOutputXMLDeviceForm ())
               {
                  DebugOutput::getErrorStream () << "XMLDeviceForm::" << __FUNCTION__ << ": iCapabilities = 0x" << std::hex << iCapabilities << std::dec << std::endl;
                  if (pbdData)
                     DebugOutput::getErrorStream () << "XMLDeviceForm::" << __FUNCTION__ << ": pbdData = " << *pbdData << std::endl;
                  else
                     DebugOutput::getErrorStream () << "XMLDeviceForm::" << __FUNCTION__ << ": pbdData is null!" << std::endl;
                  DebugOutput::getErrorStream () << "XMLDeviceForm::" << __FUNCTION__ << ": iLeft   = " << iLeft << std::endl;
                  DebugOutput::getErrorStream () << "XMLDeviceForm::" << __FUNCTION__ << ": iTop    = " << iTop << std::endl;
                  DebugOutput::getErrorStream () << "XMLDeviceForm::" << __FUNCTION__ << ": iRight  = " << iRight << std::endl;
                  DebugOutput::getErrorStream () << "XMLDeviceForm::" << __FUNCTION__ << ": iBottom = " << iBottom << std::endl;
               }
#endif

               // Create the object
               pDFRet = new XMLDeviceForm (pDevice,
                                           id,
                                           iCapabilities,
                                           pbdData,
                                           hcInfo,
                                           elmDeviceForm);

               if (  pszComma
                  && pDFRet
                  )
               {
                  int iCy = 0;

                  // @TBD
                  if (1 == sscanf (pszComma + 1, "%d", &iCy))
                  {
                     pDFRet->setCy (iCy);
                  }
               }

               if (hcInfo)
               {
                  hcInfo->setOwner (pDFRet);
               }
            }
            catch (std::string *pstringError)
            {
#ifndef RETAIL
               if (DebugOutput::shouldOutputXMLDeviceForm ()) DebugOutput::getErrorStream () << "XMLDeviceForm::" << __FUNCTION__ << ": Error: " << *pstringError << std::endl;
#endif

               delete pstringError;
            }
         }

         free ((void *)pszForm);
      }

      elmDeviceForm = nextNode (elmDeviceForm);
   }

#ifdef DEBUG
   if (DebugOutput::shouldOutputXMLDeviceForm ()) DebugOutput::getErrorStream () << "XMLDeviceForm::" << __FUNCTION__ << ": returning " << pDFRet << std::endl;
#endif

   return pDFRet;
}

PSZCRO XMLDeviceForm::
getDeviceID ()
{
   if (!pszDeviceID_d)
   {
      if (node_d)
      {
         pszDeviceID_d = getXMLContentString (node_d, node_d->doc, "deviceID");
      }
   }

#ifdef DEBUG
   if (DebugOutput::shouldOutputXMLDeviceForm ()) DebugOutput::getErrorStream () << "XMLDeviceForm::" << __FUNCTION__ << ": returning " << (pszDeviceID_d ? pszDeviceID_d : "NULL") << std::endl;
#endif

   return pszDeviceID_d;
}

bool XMLDeviceForm::
isFormSupported (int id)
{
   XMLDevice *pXMLDevice = XMLDevice::isAXMLDevice (pDevice_d);

   if (!pXMLDevice)
      return false;

   xmlDocPtr   docDeviceForms  = pXMLDevice->getDocForms ();
   xmlNodePtr  rootDeviceForms = xmlDocGetRootElement (docDeviceForms);
   xmlNodePtr  elmDeviceForm   = 0;
   bool        fFound          = false;

   if (!rootDeviceForms)
      return false;

   elmDeviceForm = firstNode (rootDeviceForms);
   if (!elmDeviceForm)
      return false;

   elmDeviceForm = firstNode (elmDeviceForm->children);

   while (  elmDeviceForm
         && !fFound
         )
   {
      PSZCRO pszForm = getXMLContentString (elmDeviceForm,
                                            docDeviceForms,
                                            "name");

      if (pszForm)
      {
         int idForm = DeviceForm::nameToID (pszForm);

         if (idForm == id)
         {
            fFound = true;
         }

         free ((void *)pszForm);
      }

      elmDeviceForm = nextNode (elmDeviceForm);
   }

   return fFound;
}

class XMLForm_Enumerator : public Enumeration
{
public:
   XMLForm_Enumerator (Device *pDevice, xmlNodePtr nodeItem)
   {
      XMLDevice *pXMLDevice = XMLDevice::isAXMLDevice (pDevice);

      pXMLDevice_d     = pXMLDevice;
      docDeviceForms_d = 0;
      nodeItem_d       = nodeItem;

      if (pXMLDevice)
      {
         docDeviceForms_d = pXMLDevice->getDocForms ();
      }
      else
      {
         nodeItem_d = 0;
      }
   }

   virtual bool hasMoreElements ()
   {
      return nodeItem_d ? true : false;
   }

   virtual void *nextElement ()
   {
      DeviceForm *pDFRet = 0;

      if (nodeItem_d)
      {
         PSZCRO pszFormName = getXMLContentString (nodeItem_d,
                                                   docDeviceForms_d,
                                                   "name");

         if (pszFormName)
         {
            pDFRet = XMLDeviceForm::create (pXMLDevice_d, pszFormName);

            free ((void *)pszFormName);
         }

         nodeItem_d = nextNode (nodeItem_d);
      }

      return pDFRet;
   }

private:
   XMLDevice  *pXMLDevice_d;
   xmlDocPtr   docDeviceForms_d;
   xmlNodePtr  nodeItem_d;
};

Enumeration * XMLDeviceForm::
getEnumeration ()
{
   XMLDevice *pXMLDevice = XMLDevice::isAXMLDevice (pDevice_d);

   if (!pXMLDevice)
      return new XMLForm_Enumerator (pDevice_d, 0);

   xmlDocPtr   docDeviceForms  = pXMLDevice->getDocForms ();
   xmlNodePtr  rootDeviceForms = xmlDocGetRootElement (docDeviceForms);
   xmlNodePtr  elmDeviceForm   = 0;

   if (!rootDeviceForms)
      return new XMLForm_Enumerator (pDevice_d, 0);

   elmDeviceForm = firstNode (rootDeviceForms);
   if (!elmDeviceForm)
      return new XMLForm_Enumerator (pDevice_d, 0);

   elmDeviceForm = firstNode (elmDeviceForm->children);

   return new XMLForm_Enumerator (pDevice_d, elmDeviceForm);
}

std::string XMLDeviceForm::
toString (std::ostringstream& oss)
{
   std::ostringstream oss2;

   oss << "{"
       << DeviceForm::toString (oss2)
       << "}";

   return oss.str ();
}

/* Provide a way to print out class data
*/
std::ostream&
operator<< (std::ostream& os, const XMLDeviceForm& const_self)
{
   XMLDeviceForm&     self = const_cast<XMLDeviceForm&>(const_self);
   std::ostringstream oss;

   os << self.toString (oss);

   return os;
}
